package Hawat::Module::Reporter;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Hawat::Module::Reporter - Short module description

=head1 SYNOPSIS

 use Hawat::Module::Reporter;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);
use File::Find;

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Value::Convertor;
use Hawat::Module;
use Mentat::Reporter::Fetcher::Default;
use Mentat::Reporter;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

# Hash containing all valid commands supported by this module and their asociated
# callback names for indirect execution.
my %COMMANDS = (
    'fetcher'   => {
        '_DESC_' => 'Operate the fetching component',
        'status' => [
            '_command_fetcher_status' ,
            'Check statistics',
            '(STRING) date - date for which to perform fetching [OPTIONAL]',
            '(STRING) abuse - abuse group for which to perform fetching [OPTIONAL]',
        ],
    },
    'engine'    => {
        '_DESC_' => 'Operate the reporting engine',
        'test' => [
            '_command_engine_test' ,
            'Test report processing and generation',
            '(STRING) date - date for which to perform fetching [OPTIONAL]',
            '(STRING) abuse - abuse group for which to perform reporting [OPTIONAL]',
        ],
    },
);

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Hawat::Module');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item get_commands() [PUBLIC]

  Usage     :
  Purpose   : Return list of all commands this module provides
  Arguments :
  Returns   :
  Throws    : Croaks, if invoked on class
  Comments  :
  See Also  :

=cut

sub get_commands()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    return %COMMANDS;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Hawat::Module::Reporter instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return $self;
}

#-----------------------------------------------------------------------

# _command_fetcher_status [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_fetcher_status
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    my ($what, $abuse) = @args;

    my $mark = $what;
    if ($mark) {
        $mark = Value::Convertor->datestr_to_unixs($mark);
    }
    else {
        $mark = int(time());
    }
    die "Invalid usage: [$0 @$chain] must receive time mark as first argument\n" unless $mark;
    my ($tl, $th) = ($mark - (86400 * 7), $mark);

    my $start = time();

    $self->out(" - Fetcher statistics for '".(scalar localtime($tl))." = '".$tl."' to '".(scalar localtime($th))." = '".$th."' (7 days):\n");

    my $fetcher = Mentat::Reporter::Fetcher::Default->new(
        filter_grammar_file => '/etc/mentat/.core/filter-grammar.conf',
        filter_schema_file  => '/etc/mentat/.core/schema-data-filter.json',
        filter_config       => '/etc/mentat/rules/filter/mentat-reporter-filter.json',
        tl                  => $tl,
        th                  => $th,
        abuse               => $abuse,
    );
    my $flist = $fetcher->fetch_abuse_list();
    $self->res->{'fal'} = $flist;
    my $glist = $fetcher->get_abuse_list();
    $self->res->{'gal'} = $glist;

    $self->out("    * fetch_abuse_list:\n");
    foreach my $item (sort @$flist) {
        $self->out("      - ".$item."\n");
    }
    $self->out("    * get_abuse_list:\n");
    foreach my $item (sort @$glist) {
        $self->out("      - ".$item."\n");
    }

    $self->res->{'tl'} = $tl;
    $self->res->{'th'} = $th;

    # Dump duration statistics
    my $fs = $fetcher->get_status();
    $self->res->{'status'} = $fs;
    $self->out("    * durations:\n");
    $self->out("      - fetching: ".Value::Convertor->duration_to_string($fs->{durations}->{fetch})."\n") if $fs->{durations}->{fetch};
    $self->out("      - prefilter: ".Value::Convertor->duration_to_string($fs->{durations}->{prefilter})."\n") if $fs->{durations}->{prefilter};
    $self->out("      - total: ".Value::Convertor->duration_to_string($fs->{durations}->{total})."\n") if $fs->{durations}->{total};

    # Dump count statistics
    $self->out("    * counts:\n");
    foreach my $item (sort keys %{$fs->{counts}}) {
        $self->out("      - ".$item.": ".$fs->{counts}->{$item}."\n");
    }

    # Dump prefilter statistics
    $self->out("    * prefilters:\n");
    foreach my $item (sort keys %{$fs->{prefilter}}) {
        $self->out("      - ".$item.": ".$fs->{prefilter}->{$item}."\n");
    }

    # Dump abuse statistics
    $self->out("    * abuses:\n");
    foreach my $item (sort keys %{$fs->{abuses}}) {
        $self->out("      - ".$item.": ".$fs->{abuses}->{$item}."\n");
    }

    # Dump template statistics
    $self->out("    * templates:\n");
    foreach my $item (sort keys %{$fs->{templates}}) {
        $self->out("      - ".$item.": ".$fs->{templates}->{$item}."\n");
    }

    # Dump severity statistics
    $self->out("    * severities:\n");
    foreach my $item (sort keys %{$fs->{severities}}) {
        $self->out("      - ".$item.": ".$fs->{severities}->{$item}."\n");
    }

    # Dump structure statistics
    $self->out("    * structure:\n");
    foreach my $item (sort keys %{$fs->{structure}}) {
        $self->out("      - ".$item.":\n");
        $self->out("        - templates:\n");
        foreach my $sitem (sort keys %{$fs->{structure}->{$item}->{templates}}) {
            $self->out("          - ".$sitem.": ".$fs->{structure}->{$item}->{templates}->{$sitem}."\n");
        }
        $self->out("        - severities:\n");
        foreach my $sitem (sort keys %{$fs->{structure}->{$item}->{severities}}) {
            $self->out("          - ".$sitem.": ".$fs->{structure}->{$item}->{severities}->{$sitem}."\n");
        }
        $self->out("        - prefilters:\n");
        foreach my $sitem (sort keys %{$fs->{structure}->{$item}->{prefilter}}) {
            $self->out("          - ".$sitem.": ".$fs->{structure}->{$item}->{prefilter}->{$sitem}."\n");
        }
        $self->out("        - postfilter: $fs->{structure}->{$item}->{postfilter}\n");
    }

    my $stop = time();
    $self->out(" - Checking duration: " . Value::Convertor->duration_to_string($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

# _command_engine_test [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  :
#  See Also  :

sub _command_engine_test
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($chain, @args) = @_;
    my ($what, $abuse) = @args;

    my $mark = $what;
    if ($mark) {
        $mark = Value::Convertor->datestr_to_unixs($mark);
    }
    else {
        $mark = int(time());
    }
    die "Invalid usage: [$0 @$chain] must receive time mark as first argument\n" unless $mark;
    my ($tl, $th) = ($mark - (86400 * 7), $mark);

    my $start = time();

    $self->out(" - Reporting for '".(scalar localtime($mark))." = '".$mark."':\n");

    my $reporter = Mentat::Reporter->new(
        filter_grammar_file => '/etc/mentat/.core/filter-grammar.conf',
        filter_schema_file  => '/etc/mentat/.core/schema-data-filter.json',
        filter_config       => '/etc/mentat/rules/filter/mentat-reporter-filter.json',
        template_dir        => '/etc/mentat/formater-templates',
        temp_folder         => '/tmp',
        charts_dir          => '/tmp',
        reports_dir         => '/tmp',
        event_database      => 'mentat',
        event_collection    => 'events',
        mail_from           => 'test@mentat.cesnet.cz',
        reply_to            => 'jan.mach@cesnet.cz',
        mail_admin          => 'jan.mach@cesnet.cz',
        mail_devel          => 'jan.mach@cesnet.cz',
        test_mode           => 1,
        template_map_e      => {},
        template_map_a      => {
            'LaBrea' => 'labrea',
            'Kippo'  => {
                'Remote login' => 'kippo-001',
            },
        },
        dict => {
            'report_extra'   => 'Extra report',
            'report_summary' => 'Souhrnný report',
            'label_e'        => 'Jiný typ útoku/problému/události',
            'label_s'        => 'Jiné typy útoků/problémů/událostí',
        },
        pilot_mode => {
            'abuse@muni.cz' => 'rt@csirt-rt-devel.ics.muni.cz ',
            'abuse@zcu.cz'  => 'abuse@zcu.cz',
            'abuse@vsb.cz'  => 'abuse@vsb.cz',
        },
        simulate => 1,
        threshold_cache_db  => 'mentat',
        threshold_cache_col => 'test_cache_threshold',
    );

    my ($process_run_log, $reports) = $reporter->report(
        time => $mark,
        groups => $abuse
    );
    $self->out('PRL: '.Dumper($process_run_log));
    $self->out('Reports: '.Dumper($reports));

    my $stop = time();
    $self->out(" - Reporting duration: " . Value::Convertor->duration_to_string($stop - $start) . " [s]\n");
    $self->res->{'duration'} = $stop - $start;
}

=pod

=back

=cut

1;
