package Mentat::Processor::Engine::Storage;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Processor::Engine::Storage - Engine performing ALERT message storing into permanent storage

=head1 SYNOPSIS

 use Mentat::Processor::Engine::Storage;

=head1 DESCRIPTION



=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

#use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#

#-- Custom application modules ------------------------------------------------#
use Log::Loger;
use Mentat::Core qw(:c_general);
use Mentat::Processor::Engine;
use Mentat::Message::Factory;
use Mentat::Storage::Factory;
use Mentat::Message::Storage::Mongo;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.1';
    $DEVEL   = 0;
    @ISA     = ('Mentat::Processor::Engine');
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item eh_process($$) [PUBLIC]

  Usage     : Emit/execute event 'message_hello' with appropriate arguments
  Purpose   :
  Arguments :
  Returns   :
  Throws    : Dies, if not invoked on object instance or given invalid arguments

=cut

sub eh_process($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($message,) = @_;
    confess "Message must be a 'Mentat::Processor::Message' implementation" unless blessed($message) and $message->isa('Mentat::Processor::Message');

    # Retrieve the ALERT message as object
    my $alert_message = $message->alert_obj;
    my $alert_class = $alert_message->class;

    my $retries = 3;
    while ($retries--) {
        eval {
            # Do not trigger signal hook
            local $SIG{__DIE__}; # See the eval documentation for explanation of this hack

            my $record_id = $self->{STORAGE}->insert_msg($alert_message);

            $self->statistics->{'cnt_message_stored'}++;
            Log::Loger->info(__PACKAGE__, "[ENG:Storage] Stored 'ALERT' message under record ID '$record_id' in the storage");

            # Send response - SUCCCESS
            $self->emit_event('message_commit', $message);
        };
        if ($@) {
            # Trap the database disconnections, perhaps the database was restarted
            if ($@ =~ /not connected/) {
                $self->statistics->{'cnt_errors_storage_reconnect'}++;
                Log::Loger->error(__PACKAGE__, "[ENG:Storage] Database connection lost, attempting to reconnect");
                $self->_reconnect();
            }
            elsif ($@ =~ /duplicate key error/) {
                $self->statistics->{'cnt_errors_storage_duplicate'}++;
                Log::Loger->error(__PACKAGE__, "[ENG:Storage] Duplicate message: '$@'");

                # Send response - Failure
                $self->emit_event('message_remove', $message);
                return EC_STOP;
            }
            else {
                $self->statistics->{'cnt_errors_storage_other'}++;
                Log::Loger->error(__PACKAGE__, "[ENG:Storage] Storage error: '$@'");

                # Send response - FAILURE
                $self->emit_event('message_banish', $message);
                return EC_STOP;
            }
        }
        else {
            last;
        }
    }

    return (EC_CONTINUE, $message);
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Mentat::Processor::Engine instance
#  Arguments :
#  Returns   :
#  Throws    : Croaks, if invoked on class
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.
#  See Also  :

sub _init
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my %config = @_;

    # Call parent version first
    $self = $self->SUPER::_init(%config);

    die "[ENG:Storage] Constructor expects configurations as argument\n" unless %config;
    die "[ENG:Storage] Storage engine configuration 'database' is missing\n" unless $config{database};
    die "[ENG:Storage] Storage engine configuration 'error_folder' is missing\n" unless $config{error_folder};

    Log::Loger->info(__PACKAGE__, "[ENG:Storage] Using storage database: '$config{database}'");
    Log::Loger->info(__PACKAGE__, "[ENG:Storage] Using folder '$config{error_folder}' as error folder");

    $self->{DATABASE_CFG} = $config{database};
    $self->{ERROR_FOLDER} = $config{error_folder};
    $self->_reconnect('first');

    return $self;
}

# _reconnect [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    :
#  Comments  :
#  See Also  :

sub _reconnect(;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($first) = @_;

    unless ($first) {
        my $backoff = 3;
        Log::Loger->notice(__PACKAGE__, "[ENG:Storage] Waiting '$backoff' second(s) before attempting to reopen database connection");
        sleep($backoff);
    }

    Log::Loger->notice(__PACKAGE__, "[ENG:Storage] (Re)Opening database connection");
    delete $self->{STORAGE};
    #$self->{STORAGE} = Mentat::Storage::Factory->storage(split(":", $self->{DATABASE_CFG}));
    $self->{STORAGE} = Mentat::Message::Storage::Mongo->new(connection => [w => 0, j => 0]);
}

=pod

=back

=cut

1;
