package Whois::Module;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Whois::Module - Base class for custom Whois modules

=head1 SYNOPSIS

 use Whois::Module;

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#
use JSON;

#-- Custom application modules ------------------------------------------------#
use Value::Convertor;
use Value::IPv4ADDR;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant MAX_AGE => 259200;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.6';
    $DEVEL   = 0;
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

=item new() [PUBLIC ,STATIC]

  Usage     : my $instance = Whois::Module->new()
  Purpose   : Create and return reference to the new instance
  Arguments : Same as the the _init() method
  Returns   : Whois::Module reference on success, croaks on failure
  Throws    : Croaks, if invoked on object
  Comments  : Internally uses _init() method, subclasses should never overload this method.
  See Also  : _init() method

=cut

sub new
{
    my $class = shift;
    confess "Class method not invoked on class" if blessed($class);
    my $self = bless ({}, $class);

    $self->_flush();

    return $self->_init(@_);
}

=item check() [PUBLIC]

  Usage     : my @warnings = $instance->check()
  Purpose   : Check the inner whois database for errors and warnings
  Arguments : None
  Returns   : ARRAY containing warnings
  Throws    : Dies, if not invoked on object instance

=cut

sub check()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    die "Whois database does not contain any data\n" unless scalar keys %{$self->{NETWORKS}};

    my @warnings = ();
    foreach my $key (keys(%{$self->{NETWORKS}})) {
        unless (exists $self->{NETWORKS}->{$key}->{'resolved_abuses'}) {
            push(@warnings, "Network '$key' does not have mandatory 'resolved_abuses' key defined'");
        }
        unless (exists $self->{NETWORKS}->{$key}->{'first'}) {
            push(@warnings, "Network '$key' does not have mandatory 'first' key defined'");
        }
        unless (exists $self->{NETWORKS}->{$key}->{'last'}) {
            push(@warnings, "Network '$key' does not have mandatory 'last' key defined'");
        }
        unless (exists $self->{NETWORKS}->{$key}->{'type'}) {
            push(@warnings, "Network '$key' does not have mandatory 'type' key defined'");
        }
        unless (exists $self->{NETWORKS}->{$key}->{'source'}) {
            push(@warnings, "Network '$key' does not have mandatory 'source' key defined'");
        }
    }
    return @warnings;
}

=item stats() [PUBLIC]

  Usage     : my @warnings = $instance->stats()
  Purpose   : Check the inner whois database for errors and warnings
  Arguments : None
  Returns   : ARRAY containing warnings
  Throws    : Dies, if not invoked on object instance

=cut

sub stats()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    my $stats = {};
    $stats->{networks}   = scalar keys %{$self->{NETWORKS}};
    $stats->{exceptions} = scalar keys %{$self->{EXCEPTIONS}};
    return $stats;
}

=item find($) [PUBLIC]

  Usage     : my $result = $instance->find('146.102.1.1');
  Purpose   : Search for given IP address in the inner whois storage
  Arguments : MIXED $ip - IP address [MANDATORY]
  Returns   : ARRAY OF HASH REFERENCES containing network structures
  Throws    : Dies, if not invoked on object instance

=cut

sub find($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ip,) = @_;

    my $ipv4obj;
    unless (blessed($ip)) {
        $ipv4obj = Value::IPv4ADDR->new($ip);
    }
    else {
        $ipv4obj = $ip;
    }
    my $ipv4num = $ipv4obj->to_numeric();

    my @networks = ();
    foreach my $network (keys(%{$self->{EXCEPTIONS}})) {
        push(@networks, $self->{EXCEPTIONS}->{$network}) if $self->_ip_in_network($ipv4num, $self->{EXCEPTIONS}->{$network});
    }
    unless (@networks) {
        foreach my $network (keys(%{$self->{NETWORKS}})) {
            push(@networks, $self->{NETWORKS}->{$network}) if $self->_ip_in_network($ipv4num, $self->{NETWORKS}->{$network});
        }
    }

    return $self->_sort_nets(@networks);
}

=item rfind($) [PUBLIC]

  Usage     : my $result = $instance->rfind('email@abuse.domain');
  Purpose   : Search for given email address in the inner whois storage
  Arguments : MIXED $email - email address [MANDATORY]
  Returns   : ARRAY OF HASH REFERENCES containing network structures
  Throws    : Dies, if not invoked on object instance

=cut

sub rfind($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($email,) = @_;

    my @networks = ();
    foreach my $network (keys(%{$self->{EXCEPTIONS}})) {
        push(@networks, $self->{EXCEPTIONS}->{$network}) if $self->_email_for_network($email, $self->{EXCEPTIONS}->{$network});
    }
    unless (@networks) {
        foreach my $network (keys(%{$self->{NETWORKS}})) {
            push(@networks, $self->{NETWORKS}->{$network}) if $self->_email_for_network($email, $self->{NETWORKS}->{$network});
        }
    }

    return $self->_sort_nets(@networks);
}

=item find_abuses($) [PUBLIC]

  Usage     : my $result = $instance->find_abuses('146.102.1.1');
  Purpose   : Search for abuse contact for given IP address in the inner whois storage
  Arguments : Same as find() method
  Returns   : STRING $abuse contact on SUCCESS, UNDEF on FAILURE
  Throws    : Dies, if not invoked on object instance
  See also  : find() method

=cut

sub find_abuses($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Check the internal whois storage
    my @results = $self->find(@_);
    if (@results) {
        my $r = shift @results;
        my @abuse_mailboxes = @{$r->{'resolved_abuses'}} if ref $r->{'resolved_abuses'};
        my $abuse_mails = join(',', @abuse_mailboxes);
        my $source = $r->{'source'};

        #Log::Loger->info(__PACKAGE__, "[NEG] Lookup result for '@_': '$abuse_mails', source: $source");
        return $abuse_mails;
    }
    return undef;
}

=item find_networks($) [PUBLIC]

  Usage     : my $result = $instance->find_networks('abuse@domain');
  Purpose   : Search for abuse contacts for given email address in the inner whois storage
  Arguments : Same as find() method
  Returns   : STRING $abuse contact on SUCCESS, UNDEF on FAILURE
  Throws    : Dies, if not invoked on object instance
  See also  : find() method

=cut

sub find_networks($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Check the internal whois storage
    my @results = $self->rfind(@_);
    if (@results) {
        my @networks = map { $_->{network} } @results;
        my $networks = join(',', @networks);

        #Log::Loger->info(__PACKAGE__, "[NEG] Reverse lookup result for '@_': '$networks'");
        return $networks;
    }
    return undef;
}

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($) [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Whois::Module instance
#  Arguments : STRING $whois_file - name of the file containing whois dump
#              BOOL $force - parse the whois dump file, even if it is too old
#  Returns   : Whois::Module reference
#  Throws    : Dies, if not invoked on object instance
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init($;$)
{
    my $self = shift;
    confess "Method needs to be implemented in subclass";
}

# _flush() [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance

sub _flush()
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    $self->{NETWORKS}   = {};
    $self->{EXCEPTIONS} = {};
}

# _set_network($$) [PROTECTED]
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance

sub _set_network($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($key, $network, $source) = @_;

    $self->{NETWORKS}->{$key} = $network;
    die "Invalid data for network '$key'\n" unless ref $self->{NETWORKS}->{$key} eq 'HASH';

    # Detect the network type
    $network->{type}   = ($network->{ip4_start}) ? 'ipv4' : 'ipv6';
    $network->{source} = $source;
}

# _set_exception($$$) [PROTECTED]
#
#  Usage     : $self->_set_exception($exception, $abuse, $source);
#  Purpose   : Populate the inner database by parsing the external whois dump file
#  Arguments : STRING $whois_file - name of the file containing whois dump
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance

sub _set_exception($$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($exception, $abuse, $source) = @_;

    my ($min, $max, $type) = $self->_parse_mmt($exception);
    $self->{EXCEPTIONS}->{$exception}->{first}           = $min;
    $self->{EXCEPTIONS}->{$exception}->{last}            = $max;
    $self->{EXCEPTIONS}->{$exception}->{source}          = 'exception:'.$source;
    $self->{EXCEPTIONS}->{$exception}->{netname}         = 'STATIC-EXCEPTION';
    $self->{EXCEPTIONS}->{$exception}->{network}         = $exception;
    $self->{EXCEPTIONS}->{$exception}->{type}            = $type;
    $self->{EXCEPTIONS}->{$exception}->{ip4_start}       = Value::Convertor->int_to_ipv4str($min);
    $self->{EXCEPTIONS}->{$exception}->{ip4_end}         = Value::Convertor->int_to_ipv4str($max);
    $self->{EXCEPTIONS}->{$exception}->{resolved_abuses} = [split(/[,; ]/, $abuse)];
}

# _parse_mmt($) [PROTECTED]
#
#  Usage     : my $result = $self->_parse_mmt($ip);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _parse_mmt($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ip,) = @_;

    my ($min, $max, $iptype);
    my ($type, @matches) = Value::Convertor->detect_ip($ip);
    if ($type eq Value::Convertor::IPV4ADDR_STRING()) {
        $min = $max = Value::Convertor->ipv4str_to_int($ip);
        $iptype = 'ipv4';
    }
    elsif ($type eq Value::Convertor::IPV4CIDR_STRING()) {
        ($min, $max) = Value::Convertor->ipv4cidr_to_ints($ip);
        $iptype = 'ipv4';
    }
    elsif ($type eq Value::Convertor::IPV4RNG_STRING()) {
        ($min, $max) = Value::Convertor->ipv4rngstr_to_ints($ip);
        $iptype = 'ipv4';
    }
    else {
        die "Unknown IP value '$ip'\n";
    }
    return ($min, $max, $iptype);
}

# _ip_in_network($$) [PROTECTED]
#
#  Usage     : my $result = $self->_ip_in_network($ip, $self->{NETWORKS}->{$network});
#  Purpose   : Check, if the given IP address is within the given IP network
#  Arguments : INTEGER $ipv4int - IP address as integer
#              HASH REFERENCE $network - network structure from inner storage
#  Returns   : INTEGER 1 on success, INTEGER 0 on failure
#  Throws    : Dies, if not invoked on object instance

sub _ip_in_network($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($ipint, $network) = @_;

    return 0 if $ipint < $network->{first};
    return 0 if $ipint > $network->{last};
    return 1;
}

# _email_for_network($$) [PROTECTED]
#
#  Usage     : my $result = $self->_email_for_network($email, $self->{NETWORKS}->{$network});
#  Purpose   : Check, if the given email address is contact
#  Arguments : STRING $email - email address as string
#              HASH REFERENCE $network - network structure from inner storage
#  Returns   : INTEGER 1 on success, INTEGER 0 on failure
#  Throws    : Dies, if not invoked on object instance

sub _email_for_network($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($email, $network) = @_;

    if (grep(/^$email$/, @{$network->{resolved_abuses}})) {
        return 1;
    }
    elsif (grep(/^$email$/, @{$network->{local_abuses}})) {
        return 1;
    }
    else {
        return 0;
    }
}

# _sort_nets(@) [PROTECTED]
#
#  Usage     : my $result = $self->_sort_nets(@networks);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _sort_nets(@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);

    return sort { $b->{'first'} <=> $a->{'first'} }
           sort { $a->{'last'} <=> $b->{'last'} } @_;
}

=pod

=back

=cut

1;
