package Whois::Negistry;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Whois::Negistry - Simple whois database constructed by parsing whois dump file

=head1 SYNOPSIS

 use Whois::Negistry;

 # Instantinate the whois database by parsing given whois dump, file must not be
 # older than three days
 my $wi_tree = Whois::Negistry->new('./data/whois.json');

 # Instantinate the whois database by parsing given whois dump, do not check the
 # file age
 my $wi_tree = Whois::Negistry->new('./data/whois.json', 1);

 # Find information regarding the given IP address
 my @results = $wi_tree->find('146.102.1.1');

 # Find information regarding the given IP address
 my $abuse = $wi_tree->find_abuse('146.102.1.1');

 # Find information regarding the given email address
 my @results = $wi_tree->rfind('abuse@domain.org');

 # Find information regarding the given email address
 my $networks = $wi_tree->find_networks('abuse@domain.org');

=head1 DESCRIPTION

Extensive module description

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.

=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Scalar::Util 'blessed';

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#
use JSON;

#-- Custom application modules ------------------------------------------------#
use Whois::Module;

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#
use constant MAX_AGE => 259200;

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL @ISA);
    $VERSION = '0.6';
    $DEVEL   = 0;
    @ISA     = ('Whois::Module');
}


#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

#-------------------------------------------------------------------------------
# Private interface
#-------------------------------------------------------------------------------

# _init($) [PROTECTED]
#
#  Usage     : return $self->_init(@_);
#  Purpose   : Initialize the new Whois::Negistry instance
#  Arguments : STRING $whois_file - name of the file containing whois dump
#              BOOL $force - parse the whois dump file, even if it is too old
#  Returns   : Whois::Negistry reference
#  Throws    : Dies, if not invoked on object instance
#  Comments  : This method must never be called directly, it is supposed to be overloaded by subclasses.

sub _init($;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($whois_file, $force, $exceptions) = @_;
    die "Negistry whois file must be given as argument" unless $whois_file;
    die "Negistry whois file '$whois_file' must be ordinary file" unless -f $whois_file;

    # Check the age of the whois dump file
    my $ts = time();
    my $ts_w = (stat($whois_file))[9];
    if (($ts - $ts_w) > MAX_AGE()) {
        unless ($force) {
            die "Negistry whois file '$whois_file' is older than 3 days\n";
        }
        elsif ($force == 1) {
            warn "Negistry whois file '$whois_file' is older than 3 days\n";
        }
    }

    # Load the whois database from given whois dump file
    $self->_read_whois_file($whois_file);

    # Read the specific exceptions
    $self->_read_exceptions($exceptions) if $exceptions;

    return $self;
}

# _read_whois_file($) [PROTECTED]
#
#  Usage     : $self->_read_whois_file($whois_file);
#  Purpose   : Populate the inner database by parsing the external whois dump file
#  Arguments : STRING $whois_file - name of the file containing whois dump
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance

sub _read_whois_file($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($whois_file,) = @_;

    # Load whois data from external file to string and untaint it
    open (my $wf, $whois_file) or die "Can`t open negistry whois file '$whois_file'\n";
    my $data = join("", <$wf>);
    close($wf);
    if ($data =~ /^(.+)$/s) { $data = $1; }

    $data = decode_json($data);
    die "Negistry whois file '$whois_file' did not contain any data\n" unless ref $data eq 'HASH';

    die "Negistry whois file '$whois_file' is in invalid format\n" unless exists $data->{"__negistry_about__"};
    delete $data->{"__negistry_about__"};

    my $k;
    foreach my $key (keys %$data) {
        $k = $key;
        $k =~ s/ //g;
        $data->{$key}->{network} = $k;
        $self->_set_network($k, $data->{$key}, 'negistry-db');
    }
}

# _read_exceptions($) [PROTECTED]
#
#  Usage     : $self->_read_exceptions($exceptions);
#  Purpose   :
#  Arguments :
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance

sub _read_exceptions($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($exceptions,) = @_;

    foreach my $exc (keys %$exceptions) {
        if (-f $exc) {
            $self->_read_exceptions_file($exc, $exceptions->{$exc});
        }
        else {
            $self->_set_exception($exc, $exceptions->{$exc}, 'manual-exception');
        }
    }
}

# _read_exceptions_file($$) [PROTECTED]
#
#  Usage     : $self->_read_exceptions_file($exceptions);
#  Purpose   : Populate the inner database by parsing the external whois dump file
#  Arguments : STRING $whois_file - name of the file containing whois dump
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance

sub _read_exceptions_file($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($file, $abuse) = @_;

    # Load message from external file to string and untaint it
    open (my $ef, $file) or die "Can`t open whois exceptions file '$file'\n";
    while (my $line = <$ef>) {
        $line =~ s/^\s+//; $line =~ s/\s+$//;
        next unless $line;
        next if ($line =~ /^\s*$/ or $line =~ /^\s*#/ or $line =~ /^\s*[a-zA-Z]+/);
        my ($ip, $rest) = split(/[\s,;]/, $line, 2);
        $self->_set_exception($ip, $abuse, $file);
    }
    close($ef);
}

=pod

=back

=cut

1;
